/////////////////////////////////////////////////////////////////////////////
// Name:        BurnHelpers.cpp
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   XX
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "BurnHelpers.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include "BurnHelpers.h"

#include <wx/process.h>

// WDR: class implementations
namespace BurnHelpers
{

  wxArrayString RunCommandCaptureOutput(const wxString& strCmd, bool bUseStdError /*= FALSE*/)
  {
    wxArrayString strReturnArray;

    wxProcess* pProcess = wxProcess::Open(strCmd);

    wxInputStream* pOutput = (bUseStdError) ? pProcess->GetErrorStream() : pProcess->GetInputStream();
      if (!pOutput)
      {
          ::wxMessageBox("Unable to get process output");
          return 1;
      }

      char chCharacter;
      wxString strLine;
      while (!pOutput->Eof())
      {
          pOutput->Read(&chCharacter, 1);
          if ((chCharacter != '\r') && (chCharacter != '\n'))
          {
              strLine += chCharacter;
          }
          else
          {
        if (!strLine.IsEmpty())
        {
                strReturnArray.Add(strLine);
                strLine = "";
        }
          }
      }
      
    return strReturnArray;
  }

  wxArrayString GetAvailableDevices()
  {
    // Run 'cdrecord -scanbus' and capture the output to enumerate the CD-ROM devices
    wxArrayString strOutputArray = BurnHelpers::RunCommandCaptureOutput("cdrecord -scanbus");
    wxArrayString strDeviceArray;

    for (unsigned int i=0; i<strOutputArray.GetCount(); i++)
    {
      wxString strLine = strOutputArray[i];
      if (strLine.Contains("Removable CD-ROM"))
      {
        // Count on the format being "        1,0,0   100) 'Lite-On ' 'LTN486  48x Max ' 'YD01' Removable CD-ROM"
        //  So we trim off the left whitespace and get the comma delimited information in the next 5 characters
        wxString strDevice = strLine.Trim(FALSE).Left(5);
        strDeviceArray.Add(strDevice);
      }
    }
    return strDeviceArray;
  }

  BurnHelpers::BurnDeviceMap GetAvailableDevicesMap()
  {
    wxArrayString DeviceArray = GetAvailableDevices();
    BurnHelpers::BurnDeviceMap ReturnDeviceMap;

    for (unsigned int i=0; i<DeviceArray.GetCount(); i++)
    {
      BurnHelpers::DeviceProperties Properties = GetDeviceProperties(DeviceArray[i]);
      ReturnDeviceMap[DeviceArray[i]] = Properties;
    }
    return ReturnDeviceMap;
  }

  BurnHelpers::DeviceProperties GetDeviceProperties(const wxString& strDevice)
  {
    // Run 'cdrecord -prcap' on the specified device and parse the output into the DeviceProperties structure
    wxArrayString strOutputArray = BurnHelpers::RunCommandCaptureOutput("cdrecord -prcap dev=" + strDevice);
    BurnHelpers::DeviceProperties Properties;
    
    Properties.m_strScsiDevice = strDevice;

    for (unsigned int i=0; i<strOutputArray.GetCount(); i++)
    {
      wxString strLine = strOutputArray[i];
      if (strLine.StartsWith("Device type    : ", &Properties.m_strDeviceType))
      {
        // Nothing to do here
      }
      else if (strLine.StartsWith("Vendor_info    : ", &Properties.m_strVendor))
      {
        Properties.m_strVendor.Replace("'", ""); // Remove the surrounding single-quotes
        Properties.m_strVendor.Trim();
      }
      else if (strLine.StartsWith("Identifikation : ", &Properties.m_strID))
      {
        Properties.m_strID.Replace("'", ""); // Remove the surrounding single-quotes
        Properties.m_strID.Trim();
      }
      else if (strLine.Matches("*Does *read CD-R media"))
      {
        Properties.m_bReadsCDR = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *read CD-RW media"))
      {
        Properties.m_bReadsCDRW = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *write CD-R media"))
      {
        Properties.m_bWritesCDR = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *write CD-RW media"))
      {
        Properties.m_bWritesCDRW = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *read DVD-ROM media"))
      {
        Properties.m_bReadsDVDROM = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *read DVD-R media"))
      {
        Properties.m_bReadsDVDR = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *read DVD-RAM media"))
      {
        Properties.m_bReadsDVDRAM = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *write DVD-R media"))
      {
        Properties.m_bWritesDVDR = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *write DVD-RAM media"))
      {
        Properties.m_bWritesDVDRAM = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *support test writing"))
      {
        Properties.m_bSupportsTestWriting = !(strLine.Contains(" not "));
      }
      else if (strLine.Matches("*Does *read multi-session CDs"))
      {
        Properties.m_bSupportsMultiSession = !(strLine.Contains(" not "));
      }
    }
    
    return Properties;
  }

  void PopulateCDCombo(BurnHelpers::BurnDeviceMap* pDeviceMap, wxComboBox* pCombo, bool bWritersOnly/* = TRUE*/)
  {
    if (pCombo)
    {
      // Loop through all the CD-ROM devices and add the ones that are writable to the combo
      pCombo->Clear();
      // Loop through all the CD-ROM devices and add them to the combo
      BurnHelpers::BurnDeviceMap::iterator start = pDeviceMap->begin();
      BurnHelpers::BurnDeviceMap::iterator finish = pDeviceMap->end();
      while (start != finish)
      {
        if (!bWritersOnly || (*start).second.m_bWritesCDR || (*start).second.m_bWritesCDRW ||
            (*start).second.m_bWritesDVDR || (*start).second.m_bWritesDVDRAM)
          pCombo->Append((*start).second.m_strVendor + " " + (*start).second.m_strID + " (" + (*start).first+ ")");

        start++;
      }
      if (pCombo->GetCount() > 0)
      {
          pCombo->SetSelection(0);
      }
    }
  }

  wxString GetDeviceFromComboString(const wxString& strComboString)
  {
    return (strComboString.AfterLast('(')).BeforeLast(')');
  }

  wxString GetBinFileName(const wxString& strFileName)
  {
    // Check if there is an extension on the file name
    if (strFileName.Contains(".") == 0)
    {
      return (strFileName + ".BIN");  // No extension, append .BIN to the end
    }
    else
    {
      wxString strFileBeforeExt = strFileName.Left(strFileName.Length()-4);
      wxString strExt = strFileName.Right(4);
      if ((strExt.CmpNoCase(".BIN") == 0) || (strExt.CmpNoCase(".CUE") == 0) || (strExt.CmpNoCase(".ISO") == 0))
      {
        return (strFileBeforeExt + ".BIN");  // No extension, append .BIN to the end
      }
      else  // Not sure what was entered, just append the .BIN
      {
        return (strFileName + ".BIN");  // No extension, append .BIN to the end
      }
    }
  }

  wxString GetCueFileName(const wxString& strFileName)
  {
    // Check if there is an extension on the file name
    if (strFileName.Contains(".") == 0)
    {
      return (strFileName + ".CUE");  // No extension, append .BIN to the end
    }
    else
    {
      wxString strFileBeforeExt = strFileName.Left(strFileName.Length()-4);
      wxString strExt = strFileName.Right(4);
      if ((strExt.CmpNoCase(".BIN") == 0) || (strExt.CmpNoCase(".CUE") == 0) || (strExt.CmpNoCase(".ISO") == 0))
      {
        return (strFileBeforeExt + ".CUE");  // No extension, append .BIN to the end
      }
      else  // Not sure what was entered, just append the .BIN
      {
        return (strFileName + ".CUE");  // No extension, append .BIN to the end
      }
    }
  }

  void PopulateCdrecordDeviceTypeCombo(wxComboBox* pCombo)
  {
    if (pCombo)
    {
      pCombo->Clear();
      pCombo->Append("mmc_cd_dvd");
      pCombo->Append("mmc_cdr");
    }
  }

  void PopulateCdrdaoDeviceTypeCombo(wxComboBox* pCombo)
  {
    if (pCombo)
    {
      pCombo->Clear();
      pCombo->Append("generic-mmc");
      pCombo->Append("generic-mmc-raw");
    }
  }

  void PopulateBurnSpeedCombo(wxComboBox* pCombo)
  {
    if (pCombo)
    {
      pCombo->Clear();
      pCombo->Append("40");
      pCombo->Append("32");
      pCombo->Append("12");
      pCombo->Append("8");
      pCombo->Append("2");
    }
  }

};


