/////////////////////////////////////////////////////////////////////////////
// Name:        BurnOptions.h
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   XX
/////////////////////////////////////////////////////////////////////////////

#ifndef __BurnOptions_H__
#define __BurnOptions_H__

#ifdef __GNUG__
    #pragma interface "BurnOptions.cpp"
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wxBurn_wdr.h"
#include "BurnHelpers.h"

//////////////////////////////////////////////////////
// Options Classes
//////////////////////////////////////////////////////
class BurnOptions : public wxObject
{
public:
  BurnOptions();

  int GetBurnSpeed() { return m_nBurnSpeed; }
  int* GetBurnSpeedPtr() { return &m_nBurnSpeed; }
  void SetBurnSpeed(int nSpeed) { m_nBurnSpeed = nSpeed; }
  wxString& GetBurnDevice() { return m_strBurnDevice; }
  wxString* GetBurnDevicePtr() { return &m_strBurnDevice; }
  void SetBurnDevice(const wxString& strBurner) { m_strBurnDevice = strBurner; }
  bool GetSimulate() { return m_bSimulate; }
  bool* GetSimulatePtr() { return &m_bSimulate; }
  void SetSimulate(bool bSimulate) { m_bSimulate = bSimulate; }
  bool GetEject() { return m_bEject; }
  bool* GetEjectPtr() { return &m_bEject; }
  void SetEject(bool bEject) { m_bEject = bEject; }
  wxString GetImageFile() { return m_strImageFile; }
  wxString* GetImageFilePtr() { return &m_strImageFile; }
  void SetImageFile(const wxString& strImageFile) { m_strImageFile = strImageFile; }
  int GetCopies() { return m_nCopies; }
  int* GetCopiesPtr() { return &m_nCopies; }
  void SetCopies(int nCopies) { m_nCopies = nCopies; }

protected:
  wxString  m_strImageFile;
  int       m_nBurnSpeed;
  wxString  m_strBurnDevice;
  bool      m_bSimulate;
  bool      m_bEject;
  int       m_nCopies;
};

class OptionsDataCD : public BurnOptions
{
public:
  OptionsDataCD();

  wxString& GetVolumeID() { return m_strVolumeID; }
  wxString* GetVolumeIDPtr() { return &m_strVolumeID; }
  void SetVolumeID(const wxString& strVolumeID) { m_strVolumeID = strVolumeID; }
  int GetFifoBuffer() { return m_nFifoBuffer; }
  int* GetFifoBufferPtr() { return &m_nFifoBuffer; }
  void SetFifoBuffer(int nFifoBuffer) { m_nFifoBuffer = nFifoBuffer; }
  bool GetFixate() { return m_bFixate; }
  bool* GetFixatePtr() { return &m_bFixate; }
  void SetFixate(bool bFixate) { m_bFixate = bFixate; }
  bool GetJoliet() { return m_bJoliet; }
  bool* GetJolietPtr() { return &m_bJoliet; }
  void SetJoliet(bool bJoliet) { m_bJoliet = bJoliet; }
  bool GetRockridge() { return m_bRockridge; }
  bool* GetRockridgePtr() { return &m_bRockridge; }
  void SetRockridge(bool bRockridge) { m_bRockridge = bRockridge; }
  bool GetKeepImages() { return m_bKeepImages; }
  bool* GetKeepImagesPtr() { return &m_bKeepImages; }
  void SetKeepImages(bool bKeepImages) { m_bKeepImages = bKeepImages; }

protected:
  // ISO Options
  wxString  m_strVolumeID;
  bool      m_bJoliet;
  bool      m_bRockridge;
  
  // CD Burner Options
  int       m_nFifoBuffer;
  bool      m_bFixate;
  bool      m_bKeepImages;
};

class OptionsVideoCD : public BurnOptions
{
public:
  OptionsVideoCD();
};

class OptionsAudioCD : public BurnOptions
{
public:
  OptionsAudioCD();
};


// WDR: class declarations

//----------------------------------------------------------------------------
// CDataOptionsDlg
//----------------------------------------------------------------------------

class CDataOptionsDlg: public wxDialog
{
public:
    // constructors and destructors
    CDataOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
        OptionsDataCD* pOptions,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxDEFAULT_DIALOG_STYLE );
    
    // WDR: method declarations for CDataOptionsDlg
    void PopulateCombo(BurnHelpers::BurnDeviceMap* pDeviceMap);
    
private:
    // WDR: member variable declarations for CDataOptionsDlg
    OptionsDataCD*  m_pOptions;
    
private:
    // WDR: handler declarations for CDataOptionsDlg
    void OnBrowse( wxCommandEvent &event );
    void OnChangeKeepCDImage( wxCommandEvent &event );

private:
    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
// CDataBurnOptionsDlg
//----------------------------------------------------------------------------

class CDataBurnOptionsDlg: public wxDialog
{
public:
    // constructors and destructors
    CDataBurnOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
        OptionsDataCD* pOptions,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxDEFAULT_DIALOG_STYLE );
    
    // WDR: method declarations for CDataBurnOptionsDlg
    void PopulateCombo(BurnHelpers::BurnDeviceMap* pDeviceMap);
    
private:
    // WDR: member variable declarations for CDataBurnOptionsDlg
    OptionsDataCD*  m_pOptions;
    
private:
    // WDR: handler declarations for CDataBurnOptionsDlg
    void OnOK( wxCommandEvent &event );
    void OnBrowseIsoFile( wxCommandEvent &event );

private:
    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
// CMakeIsoOptionsDlg
//----------------------------------------------------------------------------

class CMakeIsoOptionsDlg: public wxDialog
{
public:
    // constructors and destructors
    CMakeIsoOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
        OptionsDataCD* pOptions,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxDEFAULT_DIALOG_STYLE );
    
    // WDR: method declarations for CMakeIsoOptionsDlg
    
private:
    // WDR: member variable declarations for CMakeIsoOptionsDlg
    OptionsDataCD*  m_pOptions;
    
private:
    // WDR: handler declarations for CMakeIsoOptionsDlg
    void OnOK( wxCommandEvent &event );
    void OnBrowseIsoFile( wxCommandEvent &event );

private:
    DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
// CDataDefaultOptionsDlg
//----------------------------------------------------------------------------

class CDataDefaultOptionsDlg: public wxDialog
{
public:
    // constructors and destructors
    CDataDefaultOptionsDlg( wxWindow *parent, wxWindowID id, const wxString &title,
        OptionsDataCD* pOptions,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxDEFAULT_DIALOG_STYLE );
    
    // WDR: method declarations for CDataDefaultOptionsDlg
    void PopulateCombo(BurnHelpers::BurnDeviceMap* pDeviceMap);
    
private:
    // WDR: member variable declarations for CDataDefaultOptionsDlg
    OptionsDataCD*  m_pOptions;
    
private:
    // WDR: handler declarations for CDataDefaultOptionsDlg
    void OnBrowseIsoFile( wxCommandEvent &event );

private:
    DECLARE_EVENT_TABLE()
};




#endif
