/////////////////////////////////////////////////////////////////////////////
// Name:        BurnProgressDlg.cpp
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   XX
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "BurnProgressDlg.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include "BurnProgressDlg.h"
#include "CDCopyDialog.h"
#include "wxBurn.h"
#include "BurnHelpers.h"

#include <wx/filename.h>

int BurnProgressDlg::m_nLastReturnCode = 0;
bool BurnProgressDlg::m_bInProcess = FALSE;

// WDR: class implementations


// Here is how I plan on doing the actual burning and ISO creation
// Launch the process in question as a wxProcess with wxProcess::Open
// Turn on Redirect() to be able to get the output of the process
// Oddly enough, all output from mkisofs and cdrecord goes to stderr
// Call Read one byte at a time until a carriage return is encountered
// Parse the line and update the progress dialog accordingly


//------------------------------------------------------------------------------
// BurnProgressDlg
//------------------------------------------------------------------------------

// WDR: event table for BurnProgressDlg

BEGIN_EVENT_TABLE(BurnProgressDlg,wxDialog)
  EVT_BUTTON(ID_BUTTON_START, BurnProgressDlg::OnStart)
END_EVENT_TABLE()

int BurnProgressDlg::ProcessCmd()
{
  BurnProgressDlg::m_nLastReturnCode = 0;
  m_strOutputArray.Clear();

  wxString strCmdLine = GenerateCmdLine();
  m_strOutputArray.Add(strCmdLine);

  wxProcess* pProcess = wxProcess::Open(strCmdLine);
  if (pProcess)
  {
    BurnProgressDlg::m_bInProcess = TRUE;

    pProcess->Connect(-1, wxEVT_END_PROCESS, (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) &BurnProgressDlg::OnEndProcess);

    wxInputStream* pOutput = pProcess->GetInputStream();
    if (!pOutput)
    {
      ::wxMessageBox("Unable to get process output");
      return 1;
    }
    wxInputStream* pError = pProcess->GetErrorStream();
    if (!pError)
    {
      ::wxMessageBox("Unable to get process error");
      return 1;
    }

    char chCharacter;
    wxString strLine;
    while (!pOutput->Eof() || !pError->Eof())
    {
      while (pProcess->IsInputAvailable())
      {
        //pOutput->Read(&chCharacter, 1);
        chCharacter = pOutput->GetC();
        if ((chCharacter == '\r') || (chCharacter == '\n'))
        {
          m_strOutputArray.Add(strLine);
          printf(wxT("%s\n"), strLine.c_str());
          ProcessLine(strLine);
          strLine = "";
        }
        else
        {
          strLine += chCharacter;
        }
      }

      //Check the error stream
      while (pProcess->IsErrorAvailable())
      {
        //pError->Read(&chCharacter, 1);
        chCharacter = pError->GetC();
        if ((chCharacter == '\r') || (chCharacter == '\n'))
        {
          m_strOutputArray.Add(strLine);
          printf(wxT("%s\n"), strLine.c_str());
          ProcessLine(strLine);
          strLine = "";
        }
        else
        {
          strLine += chCharacter;
        }
      }
    }

    // Make absolutely sure that the process finishes and we have the return code
    while (BurnProgressDlg::m_bInProcess)
    {
      wxTheApp->Yield();
    }
  }

  return 0;
}

void BurnProgressDlg::ProcessLine(const wxString& strLine)
{
  if (!strLine.IsEmpty())
  {
    //wxTextCtrl* pTextCtrl = (wxTextCtrl*)FindWindow(ID_TEXTCTRL_FEEDBACK);
    wxListBox* pListBox = (wxListBox*)FindWindow(ID_LISTBOX_FEEDBACK);
    if (pListBox)
    {
      pListBox->Append(strLine);
      //pTextCtrl->AppendText(strLine + wxT("\n"));
    }
  }
  wxTheApp->Yield();
}

wxString BurnProgressDlg::ConvertToMountPoint(const wxString& strOriginalPath)
{
  wxString strMount = strOriginalPath;
  strMount.Replace(wxT("\\"), wxT("/"));
  if (strMount.Length() > 1)
  {
    if (strMount.GetChar(1) == ':')
    {
      strMount.Remove(1, 1);          // Remove the colon following the driver letter
      strMount.Prepend(wxT("//"));    // Add the double slash prefix before the drive letter on Win32
    }
  }
  return strMount;
}

void BurnProgressDlg::OnStart(wxCommandEvent& event)
{
  wxListBox* pListBox = (wxListBox*)FindWindow(ID_LISTBOX_FEEDBACK);
  if (pListBox)
  {
    pListBox->Clear();
  }

  FindWindow(ID_BUTTON_START)->Enable(FALSE);
  FindWindow(wxID_CANCEL)->Enable(FALSE);
  ProcessCmd();
  // Assume that if we're back here before the process is over then there was an error
  if (BurnProgressDlg::m_nLastReturnCode != 0)
  {
    ::wxMessageBox(wxString::Format(wxT("Error: %d"), BurnProgressDlg::m_nLastReturnCode));
  }
  FindWindow(ID_BUTTON_START)->Enable(TRUE);
  FindWindow(wxID_CANCEL)->Enable(TRUE);
}

void BurnProgressDlg::OnEndProcess(wxProcessEvent& event)
{
  // An exit code of zero (0) means okay.  Otherwise there was an error.  We need the source code of the
  //  application run to determine the error that occurred from the return code
  BurnProgressDlg::m_nLastReturnCode = event.GetExitCode();
  BurnProgressDlg::m_bInProcess = FALSE;
}

void BurnProgressDlg::DisplayErrorDialog(int nErrorCode)
{
  wxDialog ErrDlg(this, -1, "Error Dialog", wxDefaultPosition);
  ErrorDialog(&ErrDlg);
  ErrDlg.Centre();
  wxStaticText* pErrorText = (wxStaticText*)ErrDlg.FindWindow(ID_TEXT_ERROR_CODE);
  if (pErrorText)
  {
    pErrorText->SetLabel(wxString::Format(wxT("%d"), nErrorCode));
  }
  if (ErrDlg.ShowModal() == wxID_OK)
  {
    DisplayProcessLog();
  }
}

void BurnProgressDlg::DisplayProcessLog()
{
    wxDialog OutputDlg(this, -1, "Output", wxDefaultPosition);
    ProcessOutputDialog(&OutputDlg);
    OutputDlg.Centre();
    wxListBox* pOutputCtrl = (wxListBox*)OutputDlg.FindWindow(ID_LISTBOX_OUTPUT);
    if (pOutputCtrl)
    {
      pOutputCtrl->Clear();
      for (int i=0; i < m_strOutputArray.GetCount(); i++)
      {
        pOutputCtrl->Append(m_strOutputArray[i]);
      }
    }
    OutputDlg.ShowModal();
}

//----------------------------------------------------------------------------
// DataBurnProgressDlg
//----------------------------------------------------------------------------

// WDR: event table for DataBurnProgressDlg

BEGIN_EVENT_TABLE(DataBurnProgressDlg,BurnProgressDlg)
  EVT_BUTTON(ID_BUTTON_BURNCD_START, DataBurnProgressDlg::OnStart)
END_EVENT_TABLE()

void DataBurnProgressDlg::LoadDialog()
{
  BurnCDFeedback(this);

  m_pGauge = (wxGauge*)FindWindow(ID_GAUGE_BURNCD_FEEDBACK);
  if (m_pGauge)
    m_pGauge->SetRange(100);
  
  m_pCurrentOperationText = (wxStaticText*)FindWindow(ID_TEXT_BURNCD_FEEDBACK_OPERATION);
  m_pStatusText = (wxStaticText*)FindWindow(ID_TEXT_BURNCD_FEEBACK_STATUS);

  Centre();
}

void DataBurnProgressDlg::CreateISOImage(const wxString& strIsoImage/* = ""*/)
{
  m_bCreateImage = TRUE;
  m_bBurnImage = FALSE;

  SetMode(BurnProgressDlg::CREATE_IMAGE);
  LoadDialog();
  Centre();
  ShowModal();
}

void DataBurnProgressDlg::BurnISOImage(const wxString& strIsoImage/* = ""*/)
{
  m_bCreateImage = FALSE;
  m_bBurnImage = TRUE;

  SetMode(BurnProgressDlg::BURN_IMAGE);
  LoadDialog();
  Centre();
  ShowModal();
}

void DataBurnProgressDlg::BurnCDLayout()
{
  m_bCreateImage = TRUE;
  m_bBurnImage = TRUE;

  LoadDialog();

  wxASSERT(m_pOptions);
  OptionsDataCD* pOptionsDataCD = (OptionsDataCD*)m_pOptions;

  // Diagnostics
  SetMode(BurnProgressDlg::CREATE_IMAGE);
  wxString strReadCmd = GenerateCmdLine();
  SetMode(BurnProgressDlg::BURN_IMAGE);
  wxString strWriteCmd = GenerateCmdLine();
  ::wxMessageBox(wxString::Format(wxT("Read:\n%s\n\nWrite:\n%s"), strReadCmd.mb_str(), strWriteCmd.mb_str()));

  ShowModal();
}

void DataBurnProgressDlg::OnStart(wxCommandEvent& WXUNUSED(event))
{
  FindWindow(ID_BUTTON_BURNCD_START)->Enable(FALSE);
  FindWindow(wxID_OK)->Enable(FALSE);
  wxASSERT(m_pOptions);
  OptionsDataCD* pOptionsDataCD = (OptionsDataCD*)m_pOptions;

  if (m_bCreateImage)
  {
    SetMode(BurnProgressDlg::CREATE_IMAGE);
    ProcessCmd();
  }
  // Assume that if we're back here before the process is over then there was an error
  if ((BurnProgressDlg::m_nLastReturnCode == 0) || !m_bCreateImage)
  {
    if (m_bBurnImage)
    {
      for (int i = 0; i < pOptionsDataCD->GetCopies(); i++)
      {
        if (i > 0)
        {
          if (wxID_OK != ::wxMessageBox(wxT("Please insert a Blank CD"), wxT("Insert CD"), wxID_OK | wxID_CANCEL))
          {
            return;
          }
        }
        SetMode(BurnProgressDlg::BURN_IMAGE);
        ProcessCmd();
        if (BurnProgressDlg::m_nLastReturnCode != 0)
        {
          DisplayErrorDialog(BurnProgressDlg::m_nLastReturnCode);
        }
      }
    }
  }
  else
  {
    DisplayErrorDialog(BurnProgressDlg::m_nLastReturnCode);
  }

  FindWindow(ID_BUTTON_BURNCD_START)->Enable(TRUE);
  FindWindow(wxID_OK)->Enable(TRUE);
}

wxString DataBurnProgressDlg::GenerateCmdLine()
{
    wxString strCmdLine = "";
    wxASSERT(m_pOptions);
    OptionsDataCD* pOptionsDataCD = (OptionsDataCD*)m_pOptions;
    if (m_nMode == BurnProgressDlg::CREATE_IMAGE)
    {
      strCmdLine = "mkisofs -v ";
      if (pOptionsDataCD->GetImageFile() == "")
        pOptionsDataCD->SetImageFile(wxFileName::CreateTempFileName(wxT("wxBurn")));
      
      strCmdLine += " -o " + pOptionsDataCD->GetImageFile();

      if (pOptionsDataCD->GetJoliet())
        strCmdLine += " -J";

      if (pOptionsDataCD->GetRockridge())
        strCmdLine += " -R";

      if (pOptionsDataCD->GetVolumeID() != "")
        strCmdLine += " -volid " + pOptionsDataCD->GetVolumeID();

      // Now loop through the CDLayoutMap and add the mount points
      //CDLayoutTreeCtrl* pTree = (CDLayoutTreeCtrl*)FindWindow(ID_TREE_CD_DIR);
      if (m_pLayoutMap && m_pLayoutExcludeMap)
      {
        if ((m_pLayoutMap->find(wxString::Format(wxT("%c"), wxFileName::GetPathSeparator())) == m_pLayoutMap->end()) || (m_pLayoutMap->size() > 0))
          strCmdLine += " -graft-points";

        // First iterator through the exclusion map
        CDLayoutMap::iterator start = m_pLayoutExcludeMap->begin();
        CDLayoutMap::iterator end = m_pLayoutExcludeMap->end();
        while (start != end)
        {
          wxString strOrigin = BurnProgressDlg::ConvertToMountPoint((*start).second.GetOrigin());

          strCmdLine += " -exclude " + strOrigin;
          start++;
        }

        // Now go through the inclusion map
        start = m_pLayoutMap->begin();
        end = m_pLayoutMap->end();
        while (start != end)
        {
          //  mkisofs is expection the following syntax 
          //  directories:  PathOnCD/=PathOnHardDrive
          //  files:                /PathOnCD=PathOnHardDrive
          wxString strMountPoint = (*start).second.GetLayoutPosition();
          wxString strOrigin = BurnProgressDlg::ConvertToMountPoint((*start).second.GetOrigin());
          if ((*start).second.GetFileType() == CDLayoutItem::LAYOUT_DIRECTORY)
          {
            strMountPoint = strMountPoint.AfterFirst(wxFileName::GetPathSeparator()) + wxFileName::GetPathSeparator();
          }
          // Don't convert the path until here so that the above string manipulation works correctly.
          strMountPoint = BurnProgressDlg::ConvertToMountPoint(strMountPoint);

          strCmdLine += " " + strMountPoint + "=" + strOrigin;

          start++;
        }
      }
    }
    else
    {
        strCmdLine = "cdrecord -v -v ";
        if (pOptionsDataCD->GetBurnSpeed())
          strCmdLine += " speed=" + wxString::Format(wxT("%d"), pOptionsDataCD->GetBurnSpeed());

        if (pOptionsDataCD->GetBurnDevice() != "")
          strCmdLine += " dev=" + BurnHelpers::GetDeviceFromComboString(pOptionsDataCD->GetBurnDevice());
        //if (pOptionsDataCD->GetSimulate())
            strCmdLine += " -dummy";
        if (pOptionsDataCD->GetEject())
            strCmdLine += " -eject";
        if (pOptionsDataCD->GetFifoBuffer())
            strCmdLine += " fs=" + wxString::Format(wxT("%d"), pOptionsDataCD->GetFifoBuffer()) + "m";
        if (!(pOptionsDataCD->GetFixate()))
            strCmdLine += " nofix";
        if (pOptionsDataCD->GetImageFile() == "")
            pOptionsDataCD->SetImageFile(wxFileName::CreateTempFileName(wxT("wxBurn")));

        strCmdLine += " " + pOptionsDataCD->GetImageFile();
    }

    return strCmdLine;
}

void DataBurnProgressDlg::ProcessLine(const wxString& strLine)
{
  //strLine.Trim();
  // First figure out if this line tells us what action we are doing
  if (strLine.StartsWith(wxT("Starting to write")))
  {
    m_pCurrentOperationText->SetLabel(wxT("Starting Write"));
    if (m_pGauge)
      m_pGauge->SetValue(0);
  }
  else if (strLine.StartsWith(wxT("Scanning ")))
  {
    m_pCurrentOperationText->SetLabel(wxT("Scanning Source Directories"));
    if (m_pGauge)
    {
      m_pGauge->SetRange(100);
      m_pGauge->SetValue(0);
    }
  }
  else if (strLine.Contains(wxT("% done, estimate")))
  {
    double nPercent = 0;
    m_pCurrentOperationText->SetLabel(wxT("Creating Image File"));
    if (strLine.BeforeFirst('%').Trim(FALSE).Trim(TRUE).ToDouble(&nPercent))
    {
      if (m_pGauge)
        m_pGauge->SetValue((long)(floor(nPercent)));
    }
  }
  else if (strLine.Contains(wxT("extents written")))
  {
    if (m_pGauge)
      m_pGauge->SetValue(m_pGauge->GetRange());
  }
  else if (strLine.Matches("Track *: * of * MB written*"))
  {
    m_pCurrentOperationText->SetLabel(wxT("Writing"));
    // Set the gauge if this is the first MB of the track
    long int nMeg = 0;
    if (strLine.AfterFirst(':').BeforeFirst('o').Trim(FALSE).Trim(TRUE).ToLong(&nMeg))
    {
      if (nMeg == 1)
      {
        long int nTotalMeg = 0;
        if (strLine.AfterFirst('f').BeforeFirst('M').Trim(FALSE).Trim(TRUE).ToLong(&nTotalMeg))
        {
          if (m_pGauge)
            m_pGauge->SetRange(nTotalMeg);
        }
      }
      if (m_pGauge)
        m_pGauge->SetValue(nMeg);
    }
    m_pStatusText->SetLabel(strLine.BeforeFirst('(').Trim());
  }
  else if (strLine == wxT("Fixating..."))
  {
    m_pCurrentOperationText->SetLabel(wxT("Fixating"));
    m_pStatusText->SetLabel(wxT(""));
  }
  wxTheApp->Yield();
}

//----------------------------------------------------------------------------
// AudioBurnProgressDlg
//----------------------------------------------------------------------------

// WDR: event table for AudioBurnProgressDlg

BEGIN_EVENT_TABLE(AudioBurnProgressDlg,BurnProgressDlg)
END_EVENT_TABLE()

//----------------------------------------------------------------------------
// VideoBurnProgressDlg
//----------------------------------------------------------------------------

// WDR: event table for VideoBurnProgressDlg

BEGIN_EVENT_TABLE(VideoBurnProgressDlg,DaoBurnProgressDlg)
END_EVENT_TABLE()

//----------------------------------------------------------------------------
// DaoBurnProgressDlg
//----------------------------------------------------------------------------

// WDR: event table for CDCopyProgressDlg

BEGIN_EVENT_TABLE(DaoBurnProgressDlg,BurnProgressDlg)
END_EVENT_TABLE()

void DaoBurnProgressDlg::CreateBinImage(const wxString& strIsoImage /*= ""*/)
{
}

void DaoBurnProgressDlg::BurnBinImage(const wxString& strIsoImage /*= ""*/)
{
}
    
wxString DaoBurnProgressDlg::GenerateCmdLine()
{
  wxString strCmdLine = "";
  wxASSERT(m_pOptions);
  CCopyCDOptions* pOptionsCopyCD = (CCopyCDOptions*)m_pOptions;
  if (m_nMode == BurnProgressDlg::CREATE_IMAGE) // Create a BIN/CUE file via the "cdrdao read" command
  {
    strCmdLine = "cdrdao read-cd --read-raw";

    if (pOptionsCopyCD->GetSrcDevice() != "")
      strCmdLine += " --device " + BurnHelpers::GetDeviceFromComboString(pOptionsCopyCD->GetSrcDevice());
    if (pOptionsCopyCD->GetSrcDriver() != "")
      strCmdLine += " --source-driver " + pOptionsCopyCD->GetSrcDriver();

    if (pOptionsCopyCD->GetImageFile() != "")
      strCmdLine += " --datafile " + pOptionsCopyCD->GetImageFile();
  }
  else // Burn a BIN/CUE file to disk with the "cdrdao write" command
  {
    strCmdLine = "cdrdao write";

    if (pOptionsCopyCD->GetBurnSpeed())
      strCmdLine += " --speed 40";// + pOptionsCopyCD->GetBurnSpeed();
    if (pOptionsCopyCD->GetEject())
      strCmdLine += " --eject";
    if (pOptionsCopyCD->GetSimulate())
      strCmdLine += " --simulate";

    if (pOptionsCopyCD->GetBurnDevice() != "")
      strCmdLine += " --device " + BurnHelpers::GetDeviceFromComboString(pOptionsCopyCD->GetBurnDevice());
    if (pOptionsCopyCD->GetBurnDriver() != "")
      strCmdLine += " --driver " + pOptionsCopyCD->GetBurnDriver();

  }
  strCmdLine += " " + pOptionsCopyCD->GetCueFile();

  return strCmdLine;
}

//----------------------------------------------------------------------------
// CDCopyProgressDlg
//----------------------------------------------------------------------------

// WDR: event table for CDCopyProgressDlg

BEGIN_EVENT_TABLE(CDCopyProgressDlg,DaoBurnProgressDlg)
  EVT_BUTTON(ID_BUTTON_COPYCD_START, CDCopyProgressDlg::OnStart)
END_EVENT_TABLE()

void CDCopyProgressDlg::CopyCD()
{
  LoadDialog();

  m_pGauge = (wxGauge*)FindWindow(ID_GAUGE_COPYCD_FEEDBACK);
  if (m_pGauge)
    m_pGauge->SetRange(100);
  m_pCurrentOperationText = (wxStaticText*)FindWindow(ID_TEXT_COPYCD_FEEDBACK_OPERATION);
  m_pStatusText = (wxStaticText*)FindWindow(ID_TEXT_COPYCD_FEEBACK_STATUS);

  Centre();
  wxASSERT(m_pOptions);
  CCopyCDOptions* pOptionsCopyCD = (CCopyCDOptions*)m_pOptions;
  if ((pOptionsCopyCD->GetCopies() > 1) || (pOptionsCopyCD->GetKeepImages()))
  {
    SetMode(BurnProgressDlg::CREATE_IMAGE);
    wxString strReadCmd = GenerateCmdLine();
    SetMode(BurnProgressDlg::BURN_IMAGE);
    wxString strWriteCmd = GenerateCmdLine();
    ::wxMessageBox(wxString::Format(wxT("Read:\n%s\n\nWrite:\n%s"), strReadCmd.mb_str(), strWriteCmd.mb_str()));
  }
  else
  {
    wxString strCopyCmd = GenerateCmdLine();
    ::wxMessageBox(wxString::Format(wxT("Copy:\n%s"), strCopyCmd.mb_str()));
  }
  ShowModal();
}

wxString CDCopyProgressDlg::GenerateCmdLine()
{
  wxString strCmdLine = "";
  wxASSERT(m_pOptions);
  CCopyCDOptions* pOptionsCopyCD = (CCopyCDOptions*)m_pOptions;
  // Check if we are making multiple copies of the CD
  // If so, then check if we are creating the image or writing the image
  // If not, then generate a straight "cdrdao copy" command
  if ((pOptionsCopyCD->GetCopies() > 1) || (pOptionsCopyCD->GetKeepImages()))
  {
    strCmdLine = DaoBurnProgressDlg::GenerateCmdLine();
  }
  else  // Generate a straight "cdrdao copy" command
  {
    strCmdLine = wxT("cdrdao copy --read-raw");

    if (pOptionsCopyCD->GetBurnSpeed())
      strCmdLine +=  wxT(" --speed 40");// + wxString::Format(wxT("%d"), pOptionsCopyCD->GetBurnSpeed());
    if (pOptionsCopyCD->GetEject())
      strCmdLine +=  wxT(" --eject");
    if (pOptionsCopyCD->GetSimulate())
      strCmdLine +=  wxT(" --simulate");

    if (pOptionsCopyCD->GetSrcDevice() != "")
      strCmdLine +=  wxT(" --source-device ") + BurnHelpers::GetDeviceFromComboString(pOptionsCopyCD->GetSrcDevice());
    if (pOptionsCopyCD->GetBurnDevice() != "")
      strCmdLine +=  wxT(" --device ") + BurnHelpers::GetDeviceFromComboString(pOptionsCopyCD->GetBurnDevice());

    if (pOptionsCopyCD->GetSrcDriver() != "")
      strCmdLine +=  wxT(" --source-driver ") + pOptionsCopyCD->GetSrcDriver();
    if (pOptionsCopyCD->GetBurnDriver() != "")
      strCmdLine +=  wxT(" --driver ") + pOptionsCopyCD->GetBurnDriver();

    if (pOptionsCopyCD->GetKeepImages())
    {
      strCmdLine +=  wxT(" --keepimage");
      if (pOptionsCopyCD->GetImageFile())
        strCmdLine +=  wxT(" --datafile ") + BurnHelpers::GetBinFileName(pOptionsCopyCD->GetImageFile()); // BIN file
//      strCmdLine += " " + BurnHelpers::GetCueFileName(pOptionsCopyCD->GetImageFile()); // CUE file
    }
  }
  return strCmdLine;
}

void CDCopyProgressDlg::ProcessLine(const wxString& strLine)
{
  wxASSERT(m_pOptions);
  CCopyCDOptions* pOptionsCopyCD = (CCopyCDOptions*)m_pOptions;
  //strLine.Trim();
  // First figure out if this line tells us what action we are doing
  if (strLine.StartsWith("Starting"))
  {
    m_pCurrentOperationText->SetLabel(strLine);
  }
  else if (strLine.StartsWith("Copying ")) // This will start off reading the information from the CD and give us the length
  {
    m_pCurrentOperationText->SetLabel("Reading CD...");
    m_pGauge->SetValue(0);  //Start from the beginning
    // Now get the length
    int nStart = strLine.Find("length ");
    if (nStart != -1)
    {
      wxString strTime = strLine.Mid(nStart + 7).BeforeFirst(' ');  // 7 is the length of the string "length "
      wxString strMinutes = strTime.BeforeFirst(':');
      wxString strSeconds = strTime.AfterFirst(':').BeforeFirst(':');
      long nMinutes = 0;
      long nSeconds = 0;
      strMinutes.ToLong(&nMinutes);
      strSeconds.ToLong(&nSeconds);
      m_nTotalTime = (nMinutes * 60) + nSeconds;
    }
  }
  else if (strLine.Matches("*:*:*"))
  {
    wxString strMinutes = strLine.BeforeFirst(':');
    wxString strSeconds = strLine.AfterFirst(':').BeforeFirst(':');
    long nMinutes = 0;
    long nSeconds = 0;
    strMinutes.ToLong(&nMinutes);
    strSeconds.ToLong(&nSeconds);
    long nCurrentTime = (nMinutes * 60) + nSeconds;
    if (m_nTotalTime != 0)
    {
      m_pStatusText->SetLabel(wxString::Format(wxT("Reading %ld of %ld"), nCurrentTime, m_nTotalTime));
      if (m_pGauge)
      {
        int nValue = (int)(((double)nCurrentTime/(double)m_nTotalTime)*(double)100);
        m_pGauge->SetValue(nValue);
      }
    }
  }
  else if (strLine.Matches("Wrote * of * MB (Buffer *%)."))
  {
    m_pStatusText->SetLabel(strLine.BeforeFirst('B') + "B");
    wxString strCurrentAmt = strLine.AfterFirst(' ').BeforeFirst(' ');
    long nCurrentAmt = 0;
    strCurrentAmt.ToLong(&nCurrentAmt);
    if (nCurrentAmt == 1) // First visit here, notify the interface
    {
      m_pCurrentOperationText->SetLabel("Writing CD...");
      m_pGauge->SetValue(0);  //Start from the beginning
    }
    if (m_nTotalAmt == 0)
    {
      wxString strTotalAmt = strLine.AfterFirst('f').AfterFirst(' ').BeforeFirst(' ');
      strTotalAmt.ToLong(&m_nTotalAmt);
    }
    if (m_pGauge)
    {
      int nValue = (int)(((double)nCurrentAmt/(double)m_nTotalAmt)*(double)(100));
      m_pGauge->SetValue(nValue);
    }
  }
  else if (strLine.Matches("Keeping created image file \"*\"."))
  {
    pOptionsCopyCD->SetImageFile(strLine.AfterFirst('"').BeforeLast('"'));
  }
  else if (strLine.Matches("Corresponding toc-file is written to \"*\"."))
  {
    pOptionsCopyCD->SetCueFile(strLine.AfterFirst('"').BeforeLast('"'));
  }
  else
  {
    printf(wxT("Unknown: %s\n"), strLine.mb_str());
  }
  wxTheApp->Yield();
}

void CDCopyProgressDlg::OnStart(wxCommandEvent& WXUNUSED(event))
{
  FindWindow(ID_BUTTON_COPYCD_START)->Enable(FALSE);
  FindWindow(wxID_OK)->Enable(FALSE);
  CCopyCDOptions* pOptionsCopyCD = (CCopyCDOptions*)m_pOptions;
  if ((pOptionsCopyCD->GetCopies() > 1) || (pOptionsCopyCD->GetKeepImages()))
  {
    SetMode(BurnProgressDlg::CREATE_IMAGE);
    ProcessCmd();
    // Assume that if we're back here before the process is over then there was an error
    if (BurnProgressDlg::m_nLastReturnCode != 0)
    {
      ::wxMessageBox(wxString::Format(wxT("Error: %d"), BurnProgressDlg::m_nLastReturnCode));
    }
    else
    {
      for (int i = 0; i < pOptionsCopyCD->GetCopies(); i++)
      {
        if (i > 0)
        {
          if (wxID_OK != ::wxMessageBox(wxT("Please insert a Blank CD"), wxT("Insert CD"), wxID_OK | wxID_CANCEL))
          {
            return;
          }
        }
        SetMode(BurnProgressDlg::BURN_IMAGE);
        ProcessCmd();
        if (BurnProgressDlg::m_nLastReturnCode != 0)
        {
          DisplayErrorDialog(BurnProgressDlg::m_nLastReturnCode);
        }
      }
    }
  }
  else
  {
    ProcessCmd();
    // Assume that if we're back here before the process is over then there was an error
    if (BurnProgressDlg::m_nLastReturnCode != 0)
    {
      ::wxMessageBox(wxString::Format(wxT("Error: %d"), BurnProgressDlg::m_nLastReturnCode));
    }
  }
  FindWindow(ID_BUTTON_COPYCD_START)->Enable(TRUE);
  FindWindow(wxID_OK)->Enable(TRUE);
}
