/////////////////////////////////////////////////////////////////////////////
// Name:        CDCopyDialog.cpp
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   XX
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "CDCopyDialog.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include "CDCopyDialog.h"
#include "BurnHelpers.h"
#include "wxBurn.h"

#include <wx/filename.h>
#include <wx/valgen.h>

// WDR: class implementations

//----------------------------------------------------------------------------
// CCopyCDOptionsDialog
//----------------------------------------------------------------------------

// WDR: event table for CCopyCDOptionsDialog

BEGIN_EVENT_TABLE(CCopyCDOptionsDialog,wxDialog)
  EVT_CHECKBOX(ID_CHECKBOX_COPYCD_KEEP_IMAGES, CCopyCDOptionsDialog::OnChangeKeepImagesCheckbox)
  EVT_BUTTON(ID_BUTTON_COPYCD_IMAGE_FILE_BROWSE, CCopyCDOptionsDialog::OnCopyCDImageFileBrowse)
  EVT_BUTTON(ID_BUTTON_COPYCD_CUE_FILE_BROWSE, CCopyCDOptionsDialog::OnCopyCDCueFileBrowse)
  EVT_BUTTON( wxID_OK, CCopyCDOptionsDialog::OnOK )
END_EVENT_TABLE()

CCopyCDOptionsDialog::CCopyCDOptionsDialog( wxWindow *parent, wxWindowID id, const wxString &title,
    CCopyCDOptions* pOptions, const wxPoint &position, const wxSize& size, long style ) :
    wxDialog( parent, id, title, position, size, style )
{
  CopyCDOptions( this, TRUE ); 

  m_pOptions = pOptions;
  FindWindow(ID_SPINCTRL_COPYCD_FIFO)->SetValidator(wxGenericValidator(m_pOptions->GetFifoBufferPtr()));
  FindWindow(ID_CHECKBOX_COPYCD_EJECT)->SetValidator(wxGenericValidator(m_pOptions->GetEjectPtr()));
  FindWindow(ID_CHECKBOX_COPYCD_SIMULATE)->SetValidator(wxGenericValidator(m_pOptions->GetSimulatePtr()));
  FindWindow(ID_CHECKBOX_COPYCD_KEEP_IMAGES)->SetValidator(wxGenericValidator(m_pOptions->GetKeepImagesPtr()));
  FindWindow(ID_TEXTCTRL_COPYCD_IMAGE_FILE)->SetValidator(wxGenericValidator(m_pOptions->GetImageFilePtr()));
  FindWindow(ID_TEXTCTRL_COPYCD_CUE_FILE)->SetValidator(wxGenericValidator(m_pOptions->GetCueFilePtr()));

  TransferDataToWindow();

  // Turn off the image fields if the checkbox isn't marked
  if (!(m_pOptions->GetKeepImages()))
  {
    FindWindow(ID_TEXT_COPYCD_IMAGE_FILE)->Enable(FALSE);
    FindWindow(ID_TEXTCTRL_COPYCD_IMAGE_FILE)->Enable(FALSE);
    FindWindow(ID_BUTTON_COPYCD_IMAGE_FILE_BROWSE)->Enable(FALSE);
    
    FindWindow(ID_TEXT_COPYCD_CUE_FILE)->Enable(FALSE);
    FindWindow(ID_TEXTCTRL_COPYCD_CUE_FILE)->Enable(FALSE);
    FindWindow(ID_BUTTON_COPYCD_CUE_FILE_BROWSE)->Enable(FALSE);
  }
}

// WDR: handler implementations for CCopyCDOptionsDialog

void CCopyCDOptionsDialog::OnOK( wxCommandEvent &event )
{
  TransferDataFromWindow();

  if (m_pOptions->GetKeepImages())
  {
    // Make sure that a CUE file name was specified
    wxString strImage = m_pOptions->GetImageFile();
    strImage.Trim();
    if (strImage.IsEmpty())
    {
      ::wxMessageBox(wxT("Please select a BIN image file"));
      FindWindow(ID_TEXTCTRL_COPYCD_IMAGE_FILE)->SetFocus();
      return;
    }
    wxString strCue = m_pOptions->GetCueFile();
    strCue.Trim();
    if (strCue.IsEmpty())
    {
      ::wxMessageBox(wxT("Please select a CUE image file"));
      FindWindow(ID_TEXTCTRL_COPYCD_CUE_FILE)->SetFocus();
      return;
    }
  }

  EndModal(wxID_OK);
}

void CCopyCDOptionsDialog::OnChangeKeepImagesCheckbox(wxCommandEvent &event)
{
  FindWindow(ID_TEXT_COPYCD_IMAGE_FILE)->Enable(event.IsChecked());
  FindWindow(ID_TEXTCTRL_COPYCD_IMAGE_FILE)->Enable(event.IsChecked());
  FindWindow(ID_BUTTON_COPYCD_IMAGE_FILE_BROWSE)->Enable(event.IsChecked());
  
  FindWindow(ID_TEXT_COPYCD_CUE_FILE)->Enable(event.IsChecked());
  FindWindow(ID_TEXTCTRL_COPYCD_CUE_FILE)->Enable(event.IsChecked());
  FindWindow(ID_BUTTON_COPYCD_CUE_FILE_BROWSE)->Enable(event.IsChecked());
}

void CCopyCDOptionsDialog::OnCopyCDImageFileBrowse(wxCommandEvent &event)
{
  TransferDataFromWindow();
  wxString strDefaultDir = "";
  wxString strDefaultFile = "";
  wxString strWildCard = "BIN files (*.bin)|*.bin|ALL files (*.*)|*.*";
  wxFileName file(m_pOptions->GetImageFile());
  
  // Check if the text in th text control is a directory
  if (file.DirExists())
  {
    strDefaultDir = m_pOptions->GetImageFile();
    strDefaultFile = "image";
  }
  else if (file.FileExists())
  {
    strDefaultDir = m_pOptions->GetImageFile().BeforeLast(wxFileName::GetPathSeparator());
    strDefaultFile = m_pOptions->GetImageFile().AfterLast(wxFileName::GetPathSeparator());
  }
  wxFileDialog dlg(this, "Choose a file", strDefaultDir, strDefaultFile, strWildCard, wxSAVE | wxOVERWRITE_PROMPT);
  if (dlg.ShowModal() == wxID_OK)
  {
    m_pOptions->SetImageFile(dlg.GetPath());
    ((wxTextCtrl*)FindWindow(ID_TEXTCTRL_COPYCD_IMAGE_FILE))->SetValue(dlg.GetPath());
  }
  TransferDataToWindow();
}

void CCopyCDOptionsDialog::OnCopyCDCueFileBrowse(wxCommandEvent &event)
{
  TransferDataFromWindow();
  wxString strDefaultDir = "";
  wxString strDefaultFile = "";
  wxString strWildCard = "CUE files (*.cue)|*.cue|ALL files (*.*)|*.*";
  wxFileName file(m_pOptions->GetCueFile());
  
  // Check if the text in th text control is a directory
  if (file.DirExists())
  {
    strDefaultDir = m_pOptions->GetCueFile();
    strDefaultFile = "image";
  }
  else if (file.FileExists())
  {
    strDefaultDir = m_pOptions->GetCueFile().BeforeLast(wxFileName::GetPathSeparator());
    strDefaultFile = m_pOptions->GetCueFile().AfterLast(wxFileName::GetPathSeparator());
  }
  wxFileDialog dlg(this, "Choose a file", strDefaultDir, strDefaultFile, strWildCard, wxSAVE | wxOVERWRITE_PROMPT);
  if (dlg.ShowModal() == wxID_OK)
  {
    m_pOptions->SetCueFile(dlg.GetPath());
    ((wxTextCtrl*)FindWindow(ID_TEXTCTRL_COPYCD_CUE_FILE))->SetValue(dlg.GetPath());
  }
  TransferDataToWindow();
}

//----------------------------------------------------------------------------
// CopyCDOptions
//----------------------------------------------------------------------------
CCopyCDOptions::CCopyCDOptions() :
  BurnOptions()
{
  m_strSrcDevice = "";
  m_strSrcDriver = "";
  m_strBurnDriver = "";
  m_nFifoBuffer = 254;
  m_bKeepImages = FALSE;
}

//----------------------------------------------------------------------------
// CCopyCDDialog
//----------------------------------------------------------------------------

// WDR: event table for CCopyCDDialog

BEGIN_EVENT_TABLE(CCopyCDDialog,wxDialog)
  EVT_BUTTON(ID_BUTTON_COPYCD_OPTIONS, CCopyCDDialog::OnCopyCDOptions)
  EVT_BUTTON( wxID_OK, CCopyCDDialog::OnOK )
END_EVENT_TABLE()

CCopyCDDialog::CCopyCDDialog( wxWindow *parent, wxWindowID id, const wxString &title,
    /*BurnHelpers::BurnDeviceMap* pDeviceMap, */const wxPoint &position, const wxSize& size, long style ) :
    wxDialog( parent, id, title, position, size, style )
{
  CopyCDDialog( this, TRUE ); 

  // Populate the list of available drivers (and possibly default to the max speed)

  FindWindow(ID_COMBO_COPYCD_SRC_DEVICE)->SetValidator(wxGenericValidator(m_CopyCDOptions.GetSrcDevicePtr()));
  FindWindow(ID_COMBO_COPYCD_SRC_DRIVER)->SetValidator(wxGenericValidator(m_CopyCDOptions.GetSrcDriverPtr()));
  FindWindow(ID_COMBO_COPYCD_DEST_DEVICE)->SetValidator(wxGenericValidator(m_CopyCDOptions.GetBurnDevicePtr()));
  FindWindow(ID_COMBO_COPYCD_DEST_DRIVER)->SetValidator(wxGenericValidator(m_CopyCDOptions.GetBurnDriverPtr()));
  FindWindow(ID_COMBO_COPYCD_WRITE_SPEED)->SetValidator(wxGenericValidator(m_CopyCDOptions.GetBurnSpeedPtr()));
  FindWindow(ID_SPINCTRL_COPYCD_COPIES)->SetValidator(wxGenericValidator(m_CopyCDOptions.GetCopiesPtr()));
  

  TransferDataToWindow();

  PopulateComboBoxes();
}

void CCopyCDDialog::PopulateComboBoxes()
{
  BurnerFrame* pFrame = (BurnerFrame*)GetParent();
  if (pFrame)
  {
    BurnHelpers::BurnDeviceMap* pDeviceMap = pFrame->GetDeviceMap();
    if (pDeviceMap)
    {
      wxComboBox* pSrcCombo = (wxComboBox*)(FindWindow(ID_COMBO_COPYCD_SRC_DEVICE));
      BurnHelpers::PopulateCDCombo(pDeviceMap, pSrcCombo, FALSE);

      wxComboBox* pDestCombo = (wxComboBox*)(FindWindow(ID_COMBO_COPYCD_DEST_DEVICE));
      BurnHelpers::PopulateCDCombo(pDeviceMap, pDestCombo);
      TransferDataFromWindow();
    }
  }

  BurnHelpers::PopulateCdrdaoDeviceTypeCombo((wxComboBox*)FindWindow(ID_COMBO_COPYCD_SRC_DRIVER));
  BurnHelpers::PopulateCdrdaoDeviceTypeCombo((wxComboBox*)FindWindow(ID_COMBO_COPYCD_DEST_DRIVER));
  BurnHelpers::PopulateBurnSpeedCombo((wxComboBox*)FindWindow(ID_COMBO_COPYCD_WRITE_SPEED));
}

// WDR: handler implementations for CCopyCDDialog

void CCopyCDDialog::OnOK( wxCommandEvent &event )
{
  TransferDataFromWindow();

  // Make sure that a CD Source was specified
  if (m_CopyCDOptions.GetSrcDevice().IsEmpty())
  {
    ::wxMessageBox(wxT("Please select a source CD Drive"));
    FindWindow(ID_COMBO_COPYCD_SRC_DEVICE)->SetFocus();
    return;
  }
  // Make sure that a CD Burner was specified
  if (m_CopyCDOptions.GetBurnDevice().IsEmpty())
  {
    ::wxMessageBox(wxT("Please select a CD Writer"));
    FindWindow(ID_COMBO_COPYCD_DEST_DEVICE)->SetFocus();
    return;
  }

  EndModal(wxID_OK);
}
void CCopyCDDialog::OnCopyCDOptions(wxCommandEvent &event)
{
  CCopyCDOptionsDialog OptionsDlg(this, -1, "CD Copy Options", &m_CopyCDOptions);
  OptionsDlg.Centre();
  OptionsDlg.ShowModal();
}



