/////////////////////////////////////////////////////////////////////////////
// Name:        wxBurn.h
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   
/////////////////////////////////////////////////////////////////////////////

#ifndef __wxBurn_H__
#define __wxBurn_H__

#ifdef __GNUG__
    #pragma interface "wxBurn.h"
#endif

// Include wxWindows' headers

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/dir.h>

#include "wxBurn_wdr.h"
#include "CDLayoutTreeCtrl.h"
#include "wxBurnDnD.h"
#include "BurnOptions.h"
#include "BurnHelpers.h"

#define ID_TREE_AVAIL_DIR   10000
#define ID_LIST_AVAIL_FILES 10001
#define ID_TREE_CD_DIR      10002
#define ID_LIST_CD_FILES    10003
#define ID_MENU_DELETE_DIR  10004
#define ID_MENU_RENAME_DIR  10005
#define ID_MENU_CREATE_DIR  10006
#define ID_MENU_ENTER_DIR       10007
#define ID_MENU_ENTER_LAYOUT_DIR 10008
#define ID_MENU_REMOVE_FROM_LAYOUT 10009
#define ID_MENU_ADD_TREE_ITEM_TO_LAYOUT 10010
#define ID_MENU_ADD_LIST_ITEM_TO_LAYOUT 10011
// WDR: class declarations

//----------------------------------------------------------------------------
// BurnerFrame
//----------------------------------------------------------------------------

class BurnerFrame: public wxFrame
{
public:
  // constructors and destructors
  BurnerFrame( wxWindow *parent, wxWindowID id, const wxString &title,
      const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize,
      long style = wxDEFAULT_FRAME_STYLE );

  virtual ~BurnerFrame();

  CDLayoutMap* GetCDLayoutMap() { return GetCDLayoutTree()->GetLayoutMap(); }
  CDLayoutMap* GetCDLayoutExcludeMap() { return GetCDLayoutTree()->GetLayoutExcludeMap(); }
  BurnHelpers::BurnDeviceMap* GetDeviceMap() { return &m_DeviceMap; }
   
private:
  // WDR: method declarations for BurnerFrame
  wxGauge* GetGaugeDatacd()  { return (wxGauge*) FindWindow( ID_GAUGE_CD ); }
  wxListCtrl* GetCDLayoutList() { return (wxListCtrl*) FindWindow( ID_LIST_CD_FILES ); }
  CDLayoutTreeCtrl* GetCDLayoutTree() { return (CDLayoutTreeCtrl*) FindWindow( ID_TREE_CD_DIR ); }
  wxListCtrl* GetAvailFilesList() { return (wxListCtrl*) FindWindow( ID_LIST_AVAIL_FILES ); }
  wxGenericDirCtrl* GetAvailFilesTree() { return (wxGenericDirCtrl*) FindWindow( ID_TREE_AVAIL_DIR ); }
  wxSplitterWindow* GetBottomSplitterWindow()  { return (wxSplitterWindow*) FindWindow( ID_SPLITTER_BOTTOM ); }
  wxSplitterWindow* GetTopSplitterWindow()  { return (wxSplitterWindow*) FindWindow( ID_SPLITTER_TOP ); }
  wxSplitterWindow* GetSplitterDatacd()  { return (wxSplitterWindow*) FindWindow( ID_SPLITTER_DATACD ); }
  wxSplitterWindow* GetSplitterAudiocd()  { return (wxSplitterWindow*) FindWindow( ID_SPLITTER_AUDIOCD ); }
  wxSplitterWindow* GetSplitterVideocd()  { return (wxSplitterWindow*) FindWindow( ID_SPLITTER_VIDEOCD ); }
  wxNotebook* GetNotebook()  { return (wxNotebook*) FindWindow( ID_NOTEBOOK ); }
  void CreateMyMenuBar();
  void CreateMyToolBar();
  void InitializeListControls();
  void ShowMenu(wxTreeItemId id, const wxPoint& pt);

public:
    void SetUsedSpaceGauge(long nNewFileSize);
  void SetUsedSpaceProgressBar(const wxString& strNewFileSize);

private:
  // WDR: member variable declarations for BurnerFrame
  wxImageList*  m_pImageList;
  wxImageList*  m_pLayoutImageList;
  OptionsDataCD   m_DefaultDataOptions;
  OptionsVideoCD  m_DefaultVideoOptions;
  OptionsAudioCD  m_DefaultAudioOptions;
  BurnHelpers::BurnDeviceMap   m_DeviceMap;

private:
  // WDR: handler declarations for BurnerFrame
  void AvailableFilesDirChange( wxTreeEvent &event );
  void OnSelectAvailListItem( wxListEvent &event );
  void CDDirChange( wxTreeEvent &event );
  void OnBeginCDLayoutTreeLabelEdit( wxTreeEvent &event );
  void OnAbout( wxCommandEvent &event );
  void OnQuit( wxCommandEvent &event );
  void OnCloseWindow( wxCloseEvent &event );
  void OnLayoutItemRightClick(wxTreeEvent& event);
  void OnLeftDown(wxMouseEvent& event);
  void OnAvailTreeRightClick(wxTreeEvent& event);
  void OnAvailListRightClick(wxListEvent& event);
  void OnCDLayoutListRightClick(wxListEvent& event);
  void OnDragAvailList(wxListEvent& event);
  void OnDragAvailTree(wxTreeEvent& event);

  // Menu handlers
  void OnRename(wxCommandEvent& WXUNUSED(event));
  void OnCreateDirectory(wxCommandEvent& WXUNUSED(event));
  void OnDelete(wxCommandEvent& WXUNUSED(event));
  void OnEditOptionsDataCD( wxCommandEvent &event );
  void OnEditOptionsAudioCD( wxCommandEvent &event );
  void OnEditOptionsVideoCD( wxCommandEvent &event );
  void OnViewLayoutMap( wxCommandEvent &event );
  void OnViewPreviewMkisofsCmdLine( wxCommandEvent &event );
  void OnViewPreviewCdrecordCmdLine( wxCommandEvent &event );
  void OnFileNew( wxCommandEvent &event );
  void OnFileOpen( wxCommandEvent &event );
  void OnSaveLayout( wxCommandEvent &event );
  void OnSaveOptions( wxCommandEvent &event );
  void OnMakeISO( wxCommandEvent &event );
  void OnBurnISO( wxCommandEvent &event );
  void OnMakeBin( wxCommandEvent &event );
  void OnBurnBin( wxCommandEvent &event );
  void OnBurnCD( wxCommandEvent &event );
  void OnCopyCD( wxCommandEvent &event );
  void OnAddAvailTreeItemToLayout(wxCommandEvent& event);
  void OnAddAvailListItemToLayout(wxCommandEvent& event);
  void EnterListItemDir(wxCommandEvent& event);
  void EnterLayoutListItemDir(wxCommandEvent& event);
private:
  DECLARE_EVENT_TABLE()
};

//----------------------------------------------------------------------------
// BurnerApp
//----------------------------------------------------------------------------

class BurnerApp: public wxApp
{
public:
  BurnerApp();

  virtual bool OnInit();
  virtual int OnExit();
};

#endif  // __wxBurn_H__
